from lastipy.track import Track


#TODO test
class SpotifyTrack(Track):
    """Represents a track in Spotify"""

    def __init__(self, track_name, artist, spotify_id):
        self.track_name = track_name
        self.artist = artist
        self.spotify_id = spotify_id

    def __eq__(self, other):
        return isinstance(other, SpotifyTrack) \
            and self.track_name == other.track_name \
            and self.artist == other.artist \
            and self.spotify_id == self.spotify_id

    def __repr__(self):
        return str(self.__dict__)



import logging, requests
from lastipy.lastfm.library import period
from lastipy.lastfm.parse_lastfm_tracks import parse_track_name, parse_artist
from lastipy.lastfm.library.scrobbled_track import ScrobbledTrack

URL = 'http://ws.audioscrobbler.com/2.0/?method=user.gettoptracks'


def fetch_top_tracks(user, api_key, a_period=period.OVERALL):
    """Fetches the top tracks for the given user over the given period"""

    page = 1
    all_top_tracks = []
    keep_fetching = True
    logging.info("Fetching top tracks for user " + user + " over period " + a_period)
    while keep_fetching:
        json_response = _send_request(_build_json_payload(user, api_key, a_period, page))
        json_tracks = json_response['toptracks']['track']
        top_tracks = [ScrobbledTrack(parse_track_name(json_track), parse_artist(json_track), int(json_track['playcount']))
                      for json_track in json_tracks]
        
        # Filter out tracks with a playcount of 1, since those shouldn't be considered "top"
        top_tracks = [track for track in top_tracks if track.playcount > 1]
        
        logging.debug("Fetched " + str(top_tracks))
        
        all_top_tracks = all_top_tracks + top_tracks
        page = page + 1
        if not top_tracks:
            keep_fetching = False

    logging.info("Fetched " + str(len(all_top_tracks)) + " top tracks")
    logging.debug("Fetched tracks: " + str(all_top_tracks))
    return all_top_tracks

def _send_request(json_payload):
    response = requests.get(URL, params=json_payload)
    if response.ok:
        return response.json()
    else:
        response.raise_for_status()

def _build_json_payload(user, api_key, period, page):
    payload = {
        'user': user,
        'api_key': api_key,
        'format': 'json',
        'period': period,
        'page': page
    }
    return payload




import spotipy.oauth2 as oauth2
import webbrowser
import os
from lastipy import definitions

REDIRECT_URI = 'https://www.example.com/callback/'


#TODO test
def get_token(username, client_id_key, client_secret_key):
    '''Returns a Spotify token for the given user. If a cached token file exists (with the format .cache-<username>), 
    it is returned; otherwise, the given user will be prompted to authorize the app. 
    This functionw was modified from util.py in spotipy in order to expose cache path'''

    # These are the only scopes required by this app so no need to parameterize this
    scope = 'playlist-modify-public user-library-read user-library-modify user-follow-read'

    sp_oauth = oauth2.SpotifyOAuth(client_id_key,
                                   client_secret_key,
                                   REDIRECT_URI,
                                   scope=scope,
                                   cache_path=os.path.join(definitions.ROOT_DIR, '.cache-' + username))

    token_info = sp_oauth.get_cached_token()

    if not token_info:
        print('''
            User authentication requires interaction with your
            web browser. Once you enter your credentials and
            give authorization, you will be redirected to
            a url.  Paste that url you were directed to to
            complete the authorization.
        ''')
        auth_url = sp_oauth.get_authorize_url()
        try:
            webbrowser.open(auth_url)
            print("Opened %s in your browser" % auth_url)
        except:
            print("Please navigate here: %s" % auth_url)

        print()
        print()
        response = input("Enter the URL you were redirected to: ")

        print()
        print()

        code = sp_oauth.parse_response_code(response)
        token_info = sp_oauth.get_access_token(code)
    
    if token_info:
        return token_info['access_token']
    else:
        return None



import logging
import requests
from lastipy.lastfm.library.scrobbled_track import ScrobbledTrack
from requests import RequestException

URL = 'http://ws.audioscrobbler.com/2.0/?method=track.getInfo'
MAX_RETRIES = 10


def fetch_playcount(track, user, api_key):
    retries = 0
    while retries <= MAX_RETRIES:
        try:
            json_response = _send_request(_build_payload(track, user, api_key))
            break
        except RequestException:
            # This endpoint throws error 500 sometimes, so retry it a few times if it does
            if retries < MAX_RETRIES:
                logging.warn("Failed to fetch playcount for track " + track + ". Retrying...")
                retries += 1
            else:
                raise Exception("Failed to fetch playcount after " + retries + " retries")
    
    if 'track' in json_response:
        playcount = int(json_response['track']['userplaycount'])
        logging.debug(user + " has played " + str(track) + " " + str(playcount) + " times")
        return playcount
    else:
        raise Exception(json_response['error'])

def _send_request(json_payload):
    response = requests.get(URL, params=json_payload)
    if response.ok:
        return response.json()
    else:
        response.raise_for_status() 


def _build_payload(track, user, api_key):
    payload = {
        'username': user,
        'api_key': api_key,
        'format': 'json',
        'track': track.track_name,
        'artist': track.artist,
        'autocorrect': 1
    }
    return payload


def parse_track_name(json_track):
    return json_track['name']

def parse_artist(json_track):
    artist_json = json_track['artist']
    if 'name' in artist_json:
        artist = artist_json['name']
    elif '#text' in artist_json:
        artist = artist_json['#text']
    return artist
